# -*- coding: utf-8 -*-
"""
Created on Tue Mar 29 12:32:18 2016

Sample topographic shielding along transects

Martin Hurst

29/3/16

@author: mhurst
"""

#import modules
import numpy as np
import numpy.ma as ma
import gdal
#import matplotlib.pyplot as plt
import shapefile as shp

#declare shapefiles and rasters for sampling
Sites = ["hg","ss","bh"]

#Declare swath distance (in metres)
SwathDist = 2

for Site in Sites:
    
    print "Extracting transect for Site:", Site
    
    #declare files
    Transect = Site + "_transect_line.shp"
    ShieldingRasterFile = Site + "_dem2_TopoShield.bil"
    
#    try:
#        #load raster data and get properties
#        ShieldingRaster = gdal.Open(ShieldingRasterFile)
#        Projection = ShieldingRaster.GetProjection()
#        GeoTransform = ShieldingRaster.GetGeoTransform()
#        ShieldingData = ShieldingRaster.ReadAsArray()
#        NRows,NCols = np.shape(ShieldingData)
#        
#        #Setup raster for plotting in Basemap
#        DataResolution = GeoTransform[1]
#        XLL = GeoTransform[0]
#        YLL = GeoTransform[3]
#        XMin = XLL + DataResolution*0.5
#        XMax = XLL + DataResolution*ShieldingRaster.RasterXSize-DataResolution*0.5
#        YMin = YLL + DataResolution*ShieldingRaster.RasterYSize+DataResolution*0.5
#        YMax = YLL - DataResolution*0.5
#        XVector = XMin+np.arange(0,NCols)*DataResolution
#        YVector = YMin+DataResolution*np.arange(0,NRows)[::-1]
#    
#    except:
    #Get DEM properties
    import arcpy
    Description = arcpy.Describe(ShieldingRasterFile)
    Extent = Description.Extent
    DataResolution = Description.meanCellHeight
    XMin = Extent.XMin
    YMin = Extent.YMin
    YMax = Extent.YMax
    XMax = Extent.XMax

    NCols = int(Extent.width/DataResolution)
    NRows = int(Extent.height/DataResolution)

    #Convert DEM to numpy array
    #Downloaded ArcGIS 10.1 SP1 for Desktop Background Geoprocessing (64-bit) from
    #http://support.esri.com/en/downloads/patches-servicepacks/view/productid/189/metaid/1913
    ShieldingData = arcpy.RasterToNumPyArray(ShieldingRasterFile)
    XVector = XMin+np.arange(0,NCols)*DataResolution
    YVector = YMin+DataResolution*np.arange(0,NRows)[::-1]

    #mask for NDVs    
    mask = ShieldingData == -9999
    ShieldingData = ma.masked_array(ShieldingData,mask)
    
    # Open transect line and get the shapes
    sf = shp.Reader(Transect)
    Shapes = sf.shapes()
    NoLines = len(Shapes)

    #Get line points
    X1,Y1 = Shapes[0].points[0]
    X2,Y2 = Shapes[0].points[1]

    #find indices for bounding box
    #need to be careful with reverse indexing
    iStart = np.argmin(np.abs(YVector-np.max([Y1,Y2])))-1
    iEnd = np.argmin(np.abs(YVector-np.min([Y1,Y2])))+1
    jStart = np.argmin(np.abs(XVector-np.min([X1,X2])))-1
    jEnd = np.argmin(np.abs(XVector-np.max([X1,X2])))+1

    #Get Vector X and Y
    dX12 = X2-X1
    dY12 = Y2-Y1

    #Declare lists for building profiles
    XNodes = []
    YNodes = []
    DistancesAlongLine = []
    DistancesToLine = []
    
    print " -- Creating Swath Profile..."
    
    #First loop over DEM and get pixels within the swath distance
    for i in range(iStart,iEnd):

        #get Y position
        YNode = YMax-DataResolution*(i)
                
        for j in range(jStart,jEnd):

            #get X position
            XNode = XMin + j*DataResolution;
            
            
            #Get 2nd Vector Properties in Array
            dX13 = XNode-X1
            dY13 = YNode-Y1

            #Find Dot Product
            DotProduct = dX12*dX13 + dY12*dY13;

            #calculate fraction of distance along line
            t = DotProduct/(dX12*dX12 + dY12*dY12)
            if ((t < 0.) or (t > 1.)):
                continue

            #Find point along line
            XLine = X1 + t*dX12
            YLine = Y1 + t*dY12
            DistanceAlongLine = t*np.sqrt(dX12*dX12 + dY12*dY12)
            
            
            #find distance to point
            DistanceToLine = np.sqrt((XLine-XNode)*(XLine-XNode) + (YLine-YNode)*(YLine-YNode))

            if (DistanceToLine < SwathDist):
                YNodes.append(YNode)
                XNodes.append(XNode)
                DistancesAlongLine.append(DistanceAlongLine)
                DistancesToLine.append(DistanceToLine)

    print " -- Performing IDW..."
    
    #Do inverse distance weighting to generate the profile
    #Create a line for interpolating to
    LineLength = np.sqrt((X2-X1)**2 + (Y2-Y1)**2)
    NoPoints = np.int(LineLength/DataResolution)
    XLine = np.linspace(X1,X2,NoPoints)
    YLine = np.linspace(Y1,Y2,NoPoints)
    ZLine = np.zeros(len(XLine))
    DistAlongLine = np.zeros(len(XLine))
    NoPoints = len(XLine)

    #Loop along line
    for l in range(0,NoPoints):

        #Find nearest i, and j indices to DEM array
        i = np.argmin(np.abs(YLine[l]-YVector))
        j = np.argmin(np.abs(XLine[l]-XVector))

        #Calculate distance along the line, and X and Y
        DistAlongLine[l] = l*DataResolution

        #Could sample a reduced array here i.e. a neighbourhood?
        Xtv = XVector[j-2:j+3]
        Ytv = YVector[i-2:i+3]
        Xt, Yt = np.meshgrid(Xtv,Ytv)
        Shieldingt = ShieldingData[i-2:i+3, j-2:j+3]

        #skip out if we've got any NDVs in the elevation data
        if Shieldingt.mask.any():
            ZLine[l] = np.nan
            continue

        #Create a distance vector
        Dist = np.sqrt((Xt-XLine[l])**2. + (Yt-YLine[l])**2.)

        #Weights are inverse
        Weights = 1./Dist**2.

        #Interpolate Z
        mask = np.isnan(Shieldingt) == False
        ZLine[l]  = np.sum(Shieldingt[mask]*Weights[mask])/np.sum(Weights[mask])

    #Save IDW profile to a file
    File = Site + "_shielding_transect.txt"
    mask = np.isnan(ZLine) == False
    np.savetxt(File, np.transpose([XLine[mask], YLine[mask], DistAlongLine[mask], ZLine[mask]]))

    print " -- Done."
print "Done."